<?php

/**
 * Register all actions and filters for the plugin
 *
 * @link       https://www.insightengine.online
 * @since      1.0.0
 *
 * @package    Aieql
 * @subpackage Aieql/includes
 */

/**
 * Register all actions and filters for the plugin.
 *
 * Maintain a list of all hooks that are registered throughout
 * the plugin, and register them with the WordPress API. Call the
 * run function to execute the list of actions and filters.
 *
 * @package    Aieql
 * @subpackage Aieql/includes
 * @author     Insight Engine <admin@insightengine.online>
 */
class Aieql_Loader {

	/**
	 * The array of actions registered with WordPress.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $actions    The actions registered with WordPress to fire when the plugin loads.
	 */
	protected $actions;

	/**
	 * The array of filters registered with WordPress.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $filters    The filters registered with WordPress to fire when the plugin loads.
	 */
	protected $filters;

	/**
	 * Initialize the collections used to maintain the actions and filters.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->actions = array();
		$this->filters = array();

	}

	/**
	 * Add a new action to the collection to be registered with WordPress.
	 *
	 * @since    1.0.0
	 * @param    string               $hook             The name of the WordPress action that is being registered.
	 * @param    object               $component        A reference to the instance of the object on which the action is defined.
	 * @param    string               $callback         The name of the function definition on the $component.
	 * @param    int                  $priority         Optional. The priority at which the function should be fired. Default is 10.
	 * @param    int                  $accepted_args    Optional. The number of arguments that should be passed to the $callback. Default is 1.
	 */
	public function add_action( $hook, $component, $callback, $priority = 10, $accepted_args = 1 ) {
		$this->actions = $this->add( $this->actions, $hook, $component, $callback, $priority, $accepted_args );
	}

	/**
	 * Add a new filter to the collection to be registered with WordPress.
	 *
	 * @since    1.0.0
	 * @param    string               $hook             The name of the WordPress filter that is being registered.
	 * @param    object               $component        A reference to the instance of the object on which the filter is defined.
	 * @param    string               $callback         The name of the function definition on the $component.
	 * @param    int                  $priority         Optional. The priority at which the function should be fired. Default is 10.
	 * @param    int                  $accepted_args    Optional. The number of arguments that should be passed to the $callback. Default is 1
	 */
	public function add_filter( $hook, $component, $callback, $priority = 10, $accepted_args = 1 ) {
		$this->filters = $this->add( $this->filters, $hook, $component, $callback, $priority, $accepted_args );
	}

	/**
	 * A utility function that is used to register the actions and hooks into a single
	 * collection.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @param    array                $hooks            The collection of hooks that is being registered (that is, actions or filters).
	 * @param    string               $hook             The name of the WordPress filter that is being registered.
	 * @param    object               $component        A reference to the instance of the object on which the filter is defined.
	 * @param    string               $callback         The name of the function definition on the $component.
	 * @param    int                  $priority         The priority at which the function should be fired.
	 * @param    int                  $accepted_args    The number of arguments that should be passed to the $callback.
	 * @return   array                                  The collection of actions and filters registered with WordPress.
	 */
	private function add( $hooks, $hook, $component, $callback, $priority, $accepted_args ) {

		$hooks[] = array(
			'hook'          => $hook,
			'component'     => $component,
			'callback'      => $callback,
			'priority'      => $priority,
			'accepted_args' => $accepted_args
		);

		return $hooks;

	}

	/**
	 * Register the filters and actions with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {

		foreach ( $this->filters as $hook ) {
			add_filter( $hook['hook'], array( $hook['component'], $hook['callback'] ), $hook['priority'], $hook['accepted_args'] );
		}

		foreach ( $this->actions as $hook ) {
			add_action( $hook['hook'], array( $hook['component'], $hook['callback'] ), $hook['priority'], $hook['accepted_args'] );
		}

	}

}

if (!function_exists('ie_log')) {
    function ie_log($type, $data, $max_entries = 25) {
        // Check if user is logged in and get the user ID or 'guest'
        $user_id = get_current_user_id();
        $user_identifier = $user_id ? $user_id : 'guest';

        // Retrieve the current log for the specified type
        $log = get_transient('ie_log_' . $type);
        if (!$log) {
            $log = array();
        }

        // Prepend the new log entry, including the user identifier
        array_unshift($log, array(
            'time' => current_time('mysql'),
            'user' => $user_identifier,
            'data' => $data,
        ));

        // Limit the number of log entries
        $log = array_slice($log, 0, $max_entries);

        // Update the transient
        set_transient('ie_log_' . $type, $log, DAY_IN_SECONDS);
    }

	function ie_log_dashboard_widget($type) {
		// Check if the current user has the 'manage_options' capability
		if (current_user_can('manage_options')) {
			wp_add_dashboard_widget(
				'ie_log_dashboard_widget_' . $type, // Widget ID
				ucwords(str_replace('-', ' ', $type)) . ' Logs', // Title
				function() use ($type) { // Display callback
					$log = get_transient('ie_log_' . $type);
					if ($log) {
						echo '<div style="max-height:400px; overflow:auto;">';
						foreach ($log as $entry) {
							echo '<pre>';
							echo '[' . htmlspecialchars($entry['time']) . '] ';
							echo 'User: ' . htmlspecialchars($entry['user']) . ' ';
							// Use htmlspecialchars for displaying $entry['data']
							// Check if $entry['data'] is an array or object and convert it to string properly
							if (is_array($entry['data']) || is_object($entry['data'])) {
								// Convert array or object to a string representation
								$dataString = print_r($entry['data'], true);
								echo htmlspecialchars($dataString);
							} else {
								// Directly apply htmlspecialchars if it's already a string
								echo htmlspecialchars($entry['data']);
							}
							echo '</pre>';
						}
						echo '</div>';
					} else {
						echo '<p>No logs found.</p>';
					}
				}
			);
		}
	}	
}

// Hook into the 'wp_dashboard_setup' action to add our widgets
add_action('wp_dashboard_setup', function() {
    ie_log_dashboard_widget('AI Engine Query');
});

// Log AI Engine queries
add_filter( 'mwai_ai_query', function ( $query ) {
	ie_log('AI Engine Query', $query);
	return $query;
}, 999, 1 );
